# -*- coding: utf8 -*-
from __future__ import unicode_literals

import logging

from driverng import MessageV2, Worklist
from driverng.fixed import Date, FixedLengthFormat, Multiple, String

logger = logging.getLogger("driverng.bcsxp_bctmode")


class BctModeBase(FixedLengthFormat):
    pass


class Query(BctModeBase):
    FMT_DESCRIPTION = [String("message_type", 1), 1, String("sample_id", 22)]


class Order(BctModeBase):
    FMT_DESCRIPTION = [
        String("message_type", 1),
        1,
        String("sample_id", 22),
        1,
        String("emergency", 1),
        1,
        String("mode", 1),
        1,
        String("patient_name", 32),
        1,
        String("requester", 16),
        1,
        Multiple("tests", [String("code", 5), 1]),
    ]


class Result(BctModeBase):
    FMT_DESCRIPTION = [
        String("message_type", 1),
        1,
        String("sample_id", 22),
        1,
        String("assay_number", 5),
        1,
        String("result", 10),
        1,
        String("unit", 10),
        1,
        # Bit masks
        # 1: Invalid
        # 2: Doubtful
        # 4: Above measuring range
        # 8: Below measuring range
        # 16: Above reference range
        # 32: Below reference range
        # 64: Max variation exceeded
        # 128: No clot
        # 256: Repeated
        # 512: Invalid control
        # 1024: New wave
        # 2048: AMaxMin
        String("flag", 4),
        1,
        Date("date", 15, fmt_date="%y%m%d %H:%M:%S"),
        1,
        # S: Stat
        # N: Normal
        String("stat", 1),
        1,
        # U: Updated
        # N: New
        String("mode", 1),
        1,
        # M: Manually
        # H: Host
        String("source", 2),
        String("analyser", 6),
    ]


class Control(BctModeBase):
    FMT_DESCRIPTION = [
        String("message_type", 1),
        1,
        String("control_id", 4),
        1,
        String("control_lot", 17),
        1,
        String("assay_number", 5),
        1,
        String("result", 10),
        1,
        String("unit", 10),
        1,
        String("flag", 4),
        1,
        Date("date", 15, fmt_date="%y%m%d %H:%M:%S"),
        1,
        String("stat", 1),
        1,
        String("analyser", 6),
    ]


class BctModeMultiple(list):
    MESSAGE_TYPE_MAP = {"J": Order, "R": Result, "I": Query, "Q": Control}

    def add(self, message_type=None, **kwargs):
        obj = self.MESSAGE_TYPE_MAP[message_type](message_type=message_type, **kwargs)
        self.append(obj)
        return obj

    def parse(self, data):
        for line in data.splitlines():
            if line:
                obj = self.MESSAGE_TYPE_MAP[line[0]]()
                obj.parse(line)
                self.append(obj)

    def to_string(self):
        return "\n".join([e.to_string() for e in self])


class Driver(MessageV2):
    SPEC = {"code": "bcsxp_bctmode", "label": "BCS XP (BCTMODE)"}

    def create(self):
        return BctModeMultiple()

    def to_string(self, message):
        return message.to_string()

    def parse(self, data):
        obj = BctModeMultiple()
        obj.parse(data)
        return obj

    def encode_worklist(self, worklist, context):
        msg = self.create()

        for sample, action, priority in worklist.get_worklist(
            unique_together=["action", "priority"],
            priority={"A": "S", "S": "S", worklist.ELSE: "N"},
            action={"del": worklist.IGNORE, worklist.ELSE: ""},
        ):
            patient_name = "{} {}".format(sample.last_name, sample.first_name)
            message = msg.add(
                message_type="J",
                sample_id=sample.sample_id,
                emergency=priority,
                mode=action,
                patient_name=patient_name,
                requester=sample.location,
            )
            for order in sample.orders:
                message.tests.add(code=order.code)

        if msg:
            return msg

    def decode_message(self, message):
        if message and isinstance(message[0], Query):
            worklist = Worklist(context="query")
            for m in message:
                worklist.get_or_create(type="patient", sample_id=m.sample_id)
            return worklist

        worklist = Worklist(context="result")
        if message and isinstance(message[0], Control):
            for qc in message:
                lot = "{}{}".format(qc.control_id, qc.control_lot)
                sample = worklist.get_or_create(
                    sample_id=lot, control_lot=lot, type="control"
                )
                sample.results.add(
                    code=qc.assay_number.strip(), date=qc.date, value=qc.result.strip()
                )
        elif message and isinstance(message[0], Result):
            for result in message:
                sample = worklist.get_or_create(
                    sample_id=result.sample_id, type="patient"
                )
                sample.results.add(
                    code=result.assay_number.strip(),
                    date=result.date,
                    flags=[result.flag],
                    value=result.result.strip(),
                )

        return worklist
